using System;
using System.Threading;

namespace ProjectAddin
{
    [System.AddIn.AddIn("ThisProject", Version = "1.0", Publisher = "", Description = "")]
    public partial class ThisProject
    {
        //Online Container
        zenOn.IOnlineVariable MyOnlineContainer = null;

        //Queue for custom variable objects
        ZenonQueue<Variable> Queue = null; 

        private void ThisProject_Startup(object sender, EventArgs e)
        {
            //Create Eventhandlers for ThisProject Active/Inactive
            this.Active += new zenOn.ActiveEventHandler(ThisProject_Active);
            this.Inactive += new zenOn.InactiveEventHandler(ThisProject_Inactive);
        }

        void ThisProject_Active()
        {
            //Create a new queue of custom variable objects
            Queue = new ZenonQueue<Variable>();

            //Create Online Container, add two variables, create event handler
            MyOnlineContainer = this.OnlineVariables().CreateOnlineVariables("OnlineVar");
            MyOnlineContainer.Add("var1");
            MyOnlineContainer.Add("var2");
            MyOnlineContainer.Define();
            MyOnlineContainer.VariableChange += new zenOn.VariableChangeEventHandler(MyOnlineContainer_VariableChange);
        }

        void ThisProject_Inactive()
        {     
            //Undefine,defererence Online Container, release event handler
            MyOnlineContainer.Undefine();
            MyOnlineContainer.VariableChange -= new zenOn.VariableChangeEventHandler(MyOnlineContainer_VariableChange);
            MyOnlineContainer = null;
  
            //Dereference the queue 
            Queue = null;          
        }


        private void ThisProject_Shutdown(object sender, EventArgs e)
        {
            //Release event handlers
            this.Active -= new zenOn.ActiveEventHandler(ThisProject_Active);
            this.Inactive -= new zenOn.InactiveEventHandler(ThisProject_Inactive);
        }
  

        void MyOnlineContainer_VariableChange(zenOn.IVariable obVar)
        {
            //Create a new custom variable object with the information (name and value) of a (numerical) zenon variable           
            Queue.Enqueue(new Variable(obVar.Name, (double)obVar.get_Value(0)));
        }


        public void Macro_DeQueue()
        { 
            //Create a thread which dequeues all items from the queue and writes the name and value into the Immediate Window
            Thread consumerThread = new Thread(() =>
            {
                while (Queue.GetCount() > 0)
                {
                    Variable var = Queue.Dequeue();
                    System.Diagnostics.Debug.Print(var._name + ", " + var._value.ToString());
                }
            });
            consumerThread.Start();
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisProject_Startup);
            this.Shutdown += new System.EventHandler(ThisProject_Shutdown);
        }
        #endregion
    }
}
