using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;

namespace ProjectAddin
{
    //Class for custom objects, type independent, inherits from Queue
    class ZenonQueue<T> : Queue<T>
    {
        //Object for locking
        private readonly object _lock = new object();

        //Get the number of the items in the queue
        public int GetCount()
        {           
            Monitor.Enter(_lock);
            try
            {
                return base.Count;                
            }
            finally
            {
                Monitor.Exit(_lock);
            }           
        }

        //Enqueue a new item
        public new void Enqueue(T item)
        {
            Monitor.Enter(_lock);
            try
            {
                base.Enqueue(item);
                //Notify consumers that there are items to be consumed in the queue.
                Monitor.Pulse(_lock);
            }
            finally
            {
                Monitor.Exit(_lock);
            }
        }

        //Dequeue an item
        public new T Dequeue()
        {
            Monitor.Enter(_lock);
            try
            {
                while (base.Count == 0)
                    Monitor.Wait(_lock);
                //It's now safe to call Dequeue because when the call to Wait
                //returns, we own the lock (it has already been released by the caller).
                return base.Dequeue();
            }
            finally
            {
                Monitor.Exit(_lock);
            }
        }
    }
}
